-- Smart BENDer 1.1
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This tool lets you use Bend modifier in the most interactive way you can imagine
-- you can control bend center, angle and direction using two helper objects.

-- Please report any bugs or wishes by email

-- some global varialbes definition
global benddev_mesh, bendev_cent, bendev_angle, bendev_dir, bndmeshsel, bndctrlsel, bndcentsel, bndmesh, bndcent, bndctrl;

-- rollout about :)
rollout rollaboutbnd "About Smart BENDer 1.1"
(
	label ab1 "Smart BENDer 1.1"
	bitmap logobitmap filename:"sm_logo.jpg"
	label ab2 "by Jarek Dukat"
	label ab3 "http://www.nautilus.com.pl/jarek"
	label ab4 " jarek@nautilus.com.pl"
)

utility smartbender "Smart BENDer 1.1"
(
--------- utility layout
	group "Object Pickers"
	(
		label lab1 "Mesh or NURBS to Bend:"
		pickbutton bendmesh "Pick Object" filter:validmesh width:140 height:19

		label lab2 "Bend Center:" offset:[0,2]
		pickbutton bendcent "Pick Object" width:140 height:19

		label lab3 "Bend Controller:" offset:[0,2]
		pickbutton bendctrl "Pick object" width:140 height:19
	)

	group "Bend Options"
	(
		radiobuttons bendrel "Coordinate Mode:" labels:#("Relative (to mesh)","Absolute (to world)") default:1
		checkbox bendlnk1 "Link CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox bendmov1 "Move CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox bendmov2 "Move CenterObj to [0,0,0]" checked:true enabled:false offset:[-3,0]
		checkbox bendlnk2 "Link ControlObj to Mesh" checked:true offset:[-3,0]
		label lab4 "Bend Axis:" across:2 offset:[-12,0]
		radiobuttons bendax labels:#("X","Y","Z") default:3 columns:3 offset:[-6,0]
	)

	group "Bend Manager"
	(
		button bendapply "Apply Bend" offset:[0,1] height:20 width:100 enabled:false
		spinner bendscale "Bend Scale:" range:[0.01,100,1] fieldwidth:35 align:#center enabled:false
		button bendrefr "Refresh Bend" offset:[0,1] height:20 width:100 enabled:false
		button benddel "Remove Bend" tooltip:"Remove bend handler" height:20 width:100 enabled:false offset:[0,5]
		button bendfreeze "Freeze Bend" tooltip:"Create keyframes for bend modifier (if requested) and remove bend handler" height:20 width:100 enabled:false offset:[0,3]
		checkbox bendkey "Create Keyframes" checked:true enabled:false align:#center
		spinner bendkeyst "Start Frame: " range:[animationRange.start,animationRange.end,animationRange.start] fieldwidth:40 type:#integer enabled:false
		spinner bendkeyend "End Frame: " range:[animationRange.start,animationRange.end,animationRange.end] fieldwidth:40 type:#integer enabled:false
		spinner bendkeyby "Every n Frames: " range:[1,animationRange.end,5] fieldwidth:40 type:#integer enabled:false
	)

	checkbox bendexitfree "Remove Bend on Close" checked:true align:#center offset:[0,5]

--------- utility handlers
-- pick mesh object handler
	on bendmesh picked node do
	(
		bndmeshsel = true
		if bndctrlsel do
			if node.name == bndctrl.name do
			(
				messagebox "This object has already been picked for bend control,\npick another object" title:"Warning"
				bndmeshsel = false
			)
		if bndcentsel do
			if node.name == bndcent.name do
			(
				messagebox "This object has already been picked for bend center,\npick another object" title:"Warning"
				bndmeshsel = false
			)
		if bndmeshsel do
		(
			bendmesh.text = node.name
			bndmesh = node
			enablebend()
		)
	)

-- pick bend center object handler
	on bendcent picked node do
	(
		bndcentsel = true
		if bndctrlsel do
			if node.name == bndctrl.name do
			(
				messagebox "This object has already been picked for bend control,\npick another object" title:"Warning"
				bndcentsel = false
			)
		if bndmeshsel do
			if node.name == bndmesh.name do
			(
				messagebox "This object has already been picked for bending,\npick another object" title:"Warning"
				bndcentsel = false
			)
		if bndcentsel do
		(
			bendcent.text = node.name
			bndcent = node
			enablebend()
		)
	)

-- pick bend control object handler
	on bendctrl picked node do
	(
		bndctrlsel = true
		if bndcentsel do
			if node.name == bndcent.name do
			(
				messagebox "This object has already been picked for bend center,\npick another object" title:"Warning"
				bndctrlsel = false
			)
		if bndmeshsel do
			if node.name == bndmesh.name do
			(
				messagebox "This object has already been picked for bending,\npick another object" title:"Warning"
				bndctrlsel = false
			)
		if bndctrlsel do
		(
			bendctrl.text = node.name
			bndctrl = node
			enablebend()
		)
	)

-- relative/absolute handler
	on bendrel changed sel do
		if sel == 1 then
		(
			bendlnk1.enabled = bendlnk2.enabled = bendmov1.enabled = true
			bendmov2.enabled = false
		)
		else
		(
			bendlnk1.enabled = bendlnk2.enabled = bendmov1.enabled = false
			bendmov2.enabled = true
		)
	
-- keyframes on/off handler
	on bendkey changed key do
		if key then
		(
			smartbender.bendkeyst.enabled = true
			smartbender.bendkeyend.enabled = true
			smartbender.bendkeyby.enabled = true
		)
		else
		(
			smartbender.bendkeyst.enabled = false
			smartbender.bendkeyend.enabled = false
			smartbender.bendkeyby.enabled = false
		)

-- bend scale refresh handler
	on bendscale changed scl do
		if smartbender.bendrel.state == 1 then bn_refresh_angle bndctrl.pos bm:bndmesh.pos
		else bn_refresh_angle bndctrl.pos

-- bend refresh button handler
	on bendrefr pressed do refreshbend()

-- call apply bend function
	on bendapply pressed do applybend()
	
-- remove bend handler
	on benddel pressed do deletebendhandler()
	
-- freeze bend
	on bendfreeze pressed do freezebend()

-- script autostart
	on smartbender open do
	(
		bndmeshsel = false; bndcentsel = false; bndctrlsel = false
		bendev_dir = undefined; bendev_cent = undefined
		bendev_angle = undefined; benddev_mesh = undefined
		bndmesh = undefined; bndcent = undefined; bndctrl = undefined
		addrollout rollaboutbnd rolledup:true
	)

-- script close handler
	on smartbender close do
	(
		if bendexitfree.checked do deletebendhandler()
		removerollout rollaboutbnd
	)
)

fn enablebend =
	if bndmeshsel and bndcentsel and bndctrlsel do
		smartbender.bendapply.enabled = true

--------- most important part, BEND HANDLERS
fn bn_refresh_cent bc bm:[0,0,0] = bndmesh.smartbend.center = bc - bm
fn bn_refresh_angle bc bm:[0,0,0] = bndmesh.smartbend.angle = sqrt ( (bc.x - bm.x) ^ 2 + ( bc.y - bm.y ) ^ 2 ) * smartbender.bendscale.value
fn bn_refresh_dir bc bm:[0,0,0] =
	bndmesh.smartbend.direction = if (bc.x - bm.x) > 0 then
			-( asin ( (bc.y - bm.y) / sqrt ( (bc.x - bm.x) ^ 2 + (bc.y - bm.y) ^ 2)))
		else asin ( (bc.y - bm.y) / sqrt ( (bc.x - bm.x) ^ 2 + (bc.y - bm.y) ^ 2) ) + 180

-- this if a pick mesh filter
fn validmesh obj = superClassOf obj == GeometryClass

-- change buttons' state when bend handler removed
fn changebendstate =
(
	smartbender.bendapply.enabled = true
	smartbender.benddel.enabled = false
	smartbender.bendrefr.enabled = false
	smartbender.bendkey.enabled = false
	smartbender.bendfreeze.enabled = false
	smartbender.bendscale.enabled = false
	smartbender.bendkeyst.enabled = false
	smartbender.bendkeyend.enabled = false
	smartbender.bendkeyby.enabled = false
	smartbender.bendax.enabled = true
	smartbender.bendrel.enabled = true
	if smartbender.bendrel.state == 1 then
	(
		smartbender.bendlnk1.enabled = true; smartbender.bendmov1.enabled = true
		smartbender.bendmov2.enabled = false; smartbender.bendlnk2.enabled = true
	)
	else
	(
		smartbender.bendlnk1.enabled = false; smartbender.bendmov1.enabled = false
		smartbender.bendmov2.enabled = true; smartbender.bendlnk2.enabled = false
	)
)

-- refresh bend handler in current time
fn refreshbend =
(
	if smartbender.bendrel.state == 1 then
		(bn_refresh_cent bndcent.pos bm:bndmesh.pos; bn_refresh_angle bndctrl.pos bm:bndmesh.pos; bn_refresh_dir bndctrl.pos bm:bndmesh.pos)
	else
		(bn_refresh_cent bndcent.pos; bn_refresh_angle bndctrl.pos; bn_refresh_dir bndctrl.pos)
)

-- delete bend handler
fn deletebendhandler =
(
	if bendev_cent != undefined do
	(
		deletechangehandler bendev_cent
		deletechangehandler bendev_angle
		deletechangehandler bendev_dir
		deletechangehandler benddev_mesh
		deletemodifier bndmesh (bndmesh.modifiers["smartbend"])
		bendev_cent = undefined
		changebendstate()
	)
)

-- freeze bend handler
fn freezebend =
(
	deletechangehandler bendev_cent
	deletechangehandler bendev_angle
	deletechangehandler bendev_dir
	deletechangehandler benddev_mesh
	mod2 = bndmesh.modifiers["SmartBend"]
	mod2.enabledinviews = false
	befix = 1
	for i = 1 to bndmesh.modifiers.count do if (mod2 == bndmesh.modifiers[i]) do (befix = i; exit)
	addmodifier bndmesh (bend name:"FreezedSmartBend" axis:bndmesh.smartbend.axis limit:bndmesh.smartbend.limit upperlimit:bndmesh.smartbend.upperlimit lowerlimit:bndmesh.smartbend.lowerlimit) before:befix
	mod1= bndmesh.modifiers["FreezedSmartBend"]
	if smartbender.bendkey.checked then
	(
		animate on
			for tm = smartbender.bendkeyst.value to smartbender.bendkeyend.value by smartbender.bendkeyby.value do
				at time tm
				(
					slidertime = tm
					refreshbend()
					mod1.center = mod2.center
					mod1.angle = mod2.angle
					mod1.direction = mod2.direction
				)
		if smartbender.bendkeyst.value != animationRange.start do
		(
			deletekey mod1.center.keys 1
			deletekey mod1.angle.keys 1
			deletekey mod1.direction.keys 1
		)
	)
	else
	(
		mod1.center = mod2.center
		mod1.angle = mod2.angle
		mod1.direction = mod2.direction
	)
	deletemodifier bndmesh (mod2)
	bendev_cent = undefined
	changebendstate()
)

-- this function applies the bend handlers and modifier
fn applybend =
(
-- link and move helper objects depending on coordinate mode: relative/absolute
	if smartbender.bendrel.state == 1 then
	(
		if smartbender.bendmov1.checked do bndcent.pos = bndmesh.pos
		if smartbender.bendlnk1.checked do bndcent.parent = bndmesh
		if smartbender.bendlnk2.checked do bndctrl.parent = bndmesh
	)
	else if smartbender.bendmov2.checked do bndcent.pos = [0,0,0]

-- add modifier
	addmodifier bndmesh (bend name:"SmartBend" axis:(smartbender.bendax.state - 1))
	
-- apply bend handler
	if smartbender.bendrel.state == 1 then
	(
-- relative
		bn_refresh_cent bndcent.pos bm:bndmesh.pos; bn_refresh_angle bndctrl.pos bm:bndmesh.pos; bn_refresh_dir bndctrl.pos bm:bndmesh.pos
		bendev_cent = when transform bndcent changes do bn_refresh_cent bndcent.pos bm:bndmesh.pos
		bendev_angle = when transform bndctrl changes do bn_refresh_angle bndctrl.pos bm:bndmesh.pos
		bendev_dir = when transform bndctrl changes do bn_refresh_dir bndctrl.pos bm:bndmesh.pos
		benddev_mesh = when transform bndmesh changes do
		(
			bn_refresh_cent bndcent.pos bm:bndmesh.pos; bn_refresh_angle bndctrl.pos bm:bndmesh.pos; bn_refresh_dir bndctrl.pos bm:bndmesh.pos
		)
	)
	else
	(
-- absolute
		bn_refresh_cent bndcent.pos; bn_refresh_angle bndctrl.pos; bn_refresh_dir bndctrl.pos
		bendev_cent = when transform bndcent changes do bn_refresh_cent bndcent.pos
		bendev_angle = when transform bndctrl changes do bn_refresh_angle bndctrl.pos
		bendev_dir = when transform bndctrl changes do bn_refresh_dir bndctrl.pos
		benddev_mesh = when transform bndmesh changes do
		(
			bn_refresh_cent bndcent.pos; bn_refresh_angle bndctrl.pos; bn_refresh_dir bndctrl.pos
		)
	)
	
-- change buttons state
	smartbender.bendapply.enabled = false
	smartbender.bendscale.enabled = true
	smartbender.bendrefr.enabled = true
	smartbender.benddel.enabled = true
	smartbender.bendkey.enabled = true
	smartbender.bendfreeze.enabled = true
	smartbender.bendrel.enabled = false
	smartbender.bendlnk1.enabled = false; smartbender.bendmov1.enabled = false
	smartbender.bendmov2.enabled = false; smartbender.bendlnk2.enabled = false
	smartbender.bendax.enabled = false
	if smartbender.bendkey.checked do
	(
		smartbender.bendkeyst.enabled = true
		smartbender.bendkeyend.enabled = true
		smartbender.bendkeyby.enabled = true
	)
)
