-- Smart NOISEr 1.1
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This tool lets you use Noise modifier in the most interactive way you can imagine
-- you can control noise center and strength using two helper objects.

-- Please report any bugs or wishes by email

-- some global varialbes definition
global noisedev_mesh, noiseev_cent, noiseev_strength, nsmeshsel, nsctrlsel, nscentsel, nsmesh, nscent, nsctrl;

-- rollout about :)
rollout rollaboutns "About Smart NOISEr 1.1"
(
	label ab1 "Smart NOISEer 1.1"
	bitmap logobitmap filename:"sm_logo.jpg"
	label ab2 "by Jarek Dukat"
	label ab3 "http://www.nautilus.com.pl/jarek"
	label ab4 " jarek@nautilus.com.pl"
)

utility smartnoiser "Smart NOISEr 1.1"
(
--------- utility layout
	group "Object Pickers"
	(
		label lab1 "Mesh or NURBS to Noise:"
		pickbutton noisemesh "Pick Object" filter:validmesh width:140 height:19

		label lab2 "Noise Center:" offset:[0,2]
		pickbutton noisecent "Pick Object" width:140 height:19

		label lab3 "Noise Controller:" offset:[0,2]
		pickbutton noisectrl "Pick object" width:140 height:19
	)

	group "Noise Options"
	(
		radiobuttons noiserel "Coordinate Mode:" labels:#("Relative (to mesh)","Absolute (to world)") default:1
		checkbox noiselnk1 "Link CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox noisemov1 "Move CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox noisemov2 "Move CenterObj to [0,0,0]" checked:true enabled:false offset:[-3,0]
		checkbox noiselnk2 "Link ControlObj to Mesh" checked:true offset:[-3,0]
	)

	group "Noise Manager"
	(
		button noiseapply "Apply Noise" offset:[0,1] height:20 width:100 enabled:false
		spinner noisescale "Noise Scale:" range:[0.01,100,10] fieldwidth:35 scale:1 align:#center enabled:false
		spinner noiseeffect "Effect Scale:" range:[0.01,100,1] fieldwidth:35 align:#center enabled:false
		button noiserefr "Refresh Noise" offset:[0,1] height:20 width:100 enabled:false
		button noisedel "Remove Noise" tooltip:"Remove noise handler" height:20 width:100 enabled:false offset:[0,5]
		button noisefreeze "Freeze Noise" tooltip:"Create keyframes for noise modifier (if requested) and remove noise handler" height:20 width:100 enabled:false offset:[0,3]
		checkbox noisekey "Create Keyframes" checked:true enabled:false align:#center
		spinner noisekeyst "Start Frame: " range:[animationRange.start,animationRange.end,animationRange.start] fieldwidth:40 type:#integer enabled:false
		spinner noisekeyend "End Frame: " range:[animationRange.start,animationRange.end,animationRange.end] fieldwidth:40 type:#integer enabled:false
		spinner noisekeyby "Every n Frames: " range:[1,animationRange.end,5] fieldwidth:40 type:#integer enabled:false
	)

	checkbox noiseexitfree "Remove Noise on Close" checked:true align:#center offset:[0,5]

--------- utility handlers
-- pick mesh object handler
	on noisemesh picked node do
	(
		nsmeshsel = true
		if nsctrlsel do
			if node.name == nsctrl.name do
			(
				messagebox "This object has already been picked for noise control,\npick another object" title:"Warning"
				nsmeshsel = false
			)
		if nscentsel do
			if node.name == nscent.name do
			(
				messagebox "This object has already been picked for noise center,\npick another object" title:"Warning"
				nsmeshsel = false
			)
		if nsmeshsel do
		(
			noisemesh.text = node.name
			nsmesh = node
			enablenoise()
		)
	)

-- pick noise center object handler
	on noisecent picked node do
	(
		nscentsel = true
		if nsctrlsel do
			if node.name == nsctrl.name do
			(
				messagebox "This object has already been picked for noise control,\npick another object" title:"Warning"
				nscentsel = false
			)
		if nsmeshsel do
			if node.name == nsmesh.name do
			(
				messagebox "This object has already been picked for noiseing,\npick another object" title:"Warning"
				nscentsel = false
			)
		if nscentsel do
		(
			noisecent.text = node.name
			nscent = node
			enablenoise()
		)
	)

-- pick noise control object handler
	on noisectrl picked node do
	(
		nsctrlsel = true
		if nscentsel do
			if node.name == nscent.name do
			(
				messagebox "This object has already been picked for noise center,\npick another object" title:"Warning"
				nsctrlsel = false
			)
		if nsmeshsel do
			if node.name == nsmesh.name do
			(
				messagebox "This object has already been picked for noiseing,\npick another object" title:"Warning"
				nsctrlsel = false
			)
		if nsctrlsel do
		(
			noisectrl.text = node.name
			nsctrl = node
			enablenoise()
		)
	)

-- relative/absolute handler
	on noiserel changed sel do
		if sel == 1 then
		(
			noiselnk1.enabled = noiselnk2.enabled = noisemov1.enabled = true
			noisemov2.enabled = false
		)
		else
		(
			noiselnk1.enabled = noiselnk2.enabled = noisemov1.enabled = false
			noisemov2.enabled = true
		)
	
-- keyframes on/off handler
	on noisekey changed key do
		if key then
		(
			smartnoiser.noisekeyst.enabled = true
			smartnoiser.noisekeyend.enabled = true
			smartnoiser.noisekeyby.enabled = true
		)
		else
		(
			smartnoiser.noisekeyst.enabled = false
			smartnoiser.noisekeyend.enabled = false
			smartnoiser.noisekeyby.enabled = false
		)

-- noise scale refresh handler
	on noisescale changed scl do
	(
		nsmesh.smartnoise.scale = scl
		refreshnoise()
	)

-- effect scale refresh handler
	on noiseeffect changed scl do
		if smartnoiser.noiserel.state == 1 then ns_refresh_strength nsctrl.pos bm:nsmesh.pos
		else ns_refresh_strength nsctrl.pos

-- noise refresh button handler
	on noiserefr pressed do refreshnoise()

-- call apply noise function
	on noiseapply pressed do applynoise()
	
-- remove noise handler
	on noisedel pressed do deletenoisehandler()
	
-- freeze noise
	on noisefreeze pressed do freezenoise()

-- script autostart
	on smartnoiser open do
	(
		nsmeshsel = false; nscentsel = false; nsctrlsel = false
		noiseev_cent = undefined; noiseev_strength = undefined; noisedev_mesh = undefined
		nsmesh = undefined; nscent = undefined; nsctrl = undefined
		addrollout rollaboutns rolledup:true
	)

-- script close handler
	on smartnoiser close do
	(
		if noiseexitfree.checked do deletenoisehandler()
		removerollout rollaboutns
	)
)

fn enablenoise =
	if nsmeshsel and nscentsel and nsctrlsel do
		smartnoiser.noiseapply.enabled = true

--------- most important part, NOISE HANDLERS
fn ns_refresh_cent bc bm:[0,0,0] = nsmesh.smartnoise.center = bc - bm
fn ns_refresh_strength bc bm:[0,0,0] =
(
	tmpeff = smartnoiser.noiseeffect.value
	nsmesh.smartnoise.strength = [(bc.x - bm.x) * tmpeff, (bc.y - bm.y) * tmpeff, (bc.z - bm.z) * tmpeff]
)

-- this if a pick mesh filter
fn validmesh obj = superClassOf obj == GeometryClass

-- change buttons' state when noise handler removed
fn changenoisestate =
(
	smartnoiser.noiseapply.enabled = true
	smartnoiser.noisedel.enabled = false
	smartnoiser.noiserefr.enabled = false
	smartnoiser.noisekey.enabled = false
	smartnoiser.noisefreeze.enabled = false
	smartnoiser.noiseeffect.enabled = false
	smartnoiser.noisescale.enabled = false
	smartnoiser.noisekeyst.enabled = false
	smartnoiser.noisekeyend.enabled = false
	smartnoiser.noisekeyby.enabled = false
	smartnoiser.noiserel.enabled = true
	if smartnoiser.noiserel.state == 1 then
	(
		smartnoiser.noiselnk1.enabled = true; smartnoiser.noisemov1.enabled = true
		smartnoiser.noisemov2.enabled = false; smartnoiser.noiselnk2.enabled = true
	)
	else
	(
		smartnoiser.noiselnk1.enabled = false; smartnoiser.noisemov1.enabled = false
		smartnoiser.noisemov2.enabled = true; smartnoiser.noiselnk2.enabled = false
	)
)

-- refresh noise handler in current time
fn refreshnoise =
(
	if smartnoiser.noiserel.state == 1 then
		(ns_refresh_cent nscent.pos bm:nsmesh.pos; ns_refresh_strength nsctrl.pos bm:nsmesh.pos)
	else
		(ns_refresh_cent nscent.pos; ns_refresh_strength nsctrl.pos)
)

-- delete noise handler
fn deletenoisehandler =
(
	if noiseev_cent != undefined do
	(
		deletechangehandler noiseev_cent
		deletechangehandler noiseev_strength
		deletechangehandler noisedev_mesh
		deletemodifier nsmesh (nsmesh.modifiers["smartnoise"])
		noiseev_cent = undefined
		changenoisestate()
		smartnoiser.noisescale.value = 10
	)
)

-- freeze noise handler
fn freezenoise =
(
	deletechangehandler noiseev_cent
	deletechangehandler noiseev_strength
	deletechangehandler noisedev_mesh
	mod2= nsmesh.modifiers["SmartNoise"]
	mod2.enabledinviews = false
	befix = 1
	for i = 1 to nsmesh.modifiers.count do if (mod2 == nsmesh.modifiers[i]) do (befix = i; exit)
	addmodifier nsmesh (noisemodifier name:"FreezedSmartNoise" animate:nsmesh.smartnoise.animate \
		scale:nsmesh.smartnoise.scale seed:nsmesh.smartnoise.seed frequency:nsmesh.smartnoise.frequency \
		fractal:nsmesh.smartnoise.fractal roughness:nsmesh.smartnoise.roughness \
		iterations:nsmesh.smartnoise.iterations) before:befix
	mod1= nsmesh.modifiers["FreezedSmartNoise"]
	if smartnoiser.noisekey.checked then
	(
		animate on
			for tm = smartnoiser.noisekeyst.value to smartnoiser.noisekeyend.value by smartnoiser.noisekeyby.value do
				at time tm
				(
					slidertime = tm
					refreshnoise()
					mod1.center = mod2.center
					mod1.strength = mod2.strength
				)
		if smartnoiser.noisekeyst.value != animationRange.start do
		(
			deletekey mod1.center.keys 1
			deletekey mod1.strength.keys 1
		)
	)
	else
	(
		mod1.center = mod2.center
		mod1.strength = mod2.strength
	)
	deletemodifier nsmesh (mod2)
	noiseev_cent = undefined
	changenoisestate()
)

-- this function applies the noise handlers and modifier
fn applynoise =
(
-- link and move helper objects depending on coordinate mode: relative/absolute
	if smartnoiser.noiserel.state == 1 then
	(
		if smartnoiser.noisemov1.checked do nscent.pos = nsmesh.pos
		if smartnoiser.noiselnk1.checked do nscent.parent = nsmesh
		if smartnoiser.noiselnk2.checked do nsctrl.parent = nsmesh
	)
	else if smartnoiser.noisemov2.checked do nscent.pos = [0,0,0]

-- add modifier
	addmodifier nsmesh (noisemodifier name:"SmartNoise" scale:10 seed:(random 0 32000))
	
-- apply noise handler
	if smartnoiser.noiserel.state == 1 then
	(
-- relative
		ns_refresh_cent nscent.pos bm:nsmesh.pos; ns_refresh_strength nsctrl.pos bm:nsmesh.pos
		noiseev_cent = when transform nscent changes do ns_refresh_cent nscent.pos bm:nsmesh.pos
		noiseev_strength = when transform nsctrl changes do ns_refresh_strength nsctrl.pos bm:nsmesh.pos
		noisedev_mesh = when transform nsmesh changes do
		(
			ns_refresh_cent nscent.pos bm:nsmesh.pos; ns_refresh_strength nsctrl.pos bm:nsmesh.pos
		)
	)
	else
	(
-- absolute
		ns_refresh_cent nscent.pos; ns_refresh_strength nsctrl.pos
		noiseev_cent = when transform nscent changes do ns_refresh_cent nscent.pos
		noiseev_strength = when transform nsctrl changes do ns_refresh_strength nsctrl.pos
		noisedev_mesh = when transform nsmesh changes do
		(
			ns_refresh_cent nscent.pos; ns_refresh_strength nsctrl.pos
		)
	)
	
-- change buttons state
	smartnoiser.noiseapply.enabled = false
	smartnoiser.noiseeffect.enabled = true
	smartnoiser.noisescale.enabled = true
	smartnoiser.noiserefr.enabled = true
	smartnoiser.noisedel.enabled = true
	smartnoiser.noisekey.enabled = true
	smartnoiser.noisefreeze.enabled = true
	smartnoiser.noiserel.enabled = false
	smartnoiser.noiselnk1.enabled = false; smartnoiser.noisemov1.enabled = false
	smartnoiser.noisemov2.enabled = false; smartnoiser.noiselnk2.enabled = false
	if smartnoiser.noisekey.checked do
	(
		smartnoiser.noisekeyst.enabled = true
		smartnoiser.noisekeyend.enabled = true
		smartnoiser.noisekeyby.enabled = true
	)
)
