-- Smart SKEWer 1.1
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This tool lets you use Skew modifier in the most interactive way you can imagine
-- you can control skew center, amount and direction using two helper objects.

-- Please report any bugs or wishes by email

-- some global varialbes definition
global skewdev_mesh, skewev_cent, skewev_amount, skewev_dir, skmeshsel, skctrlsel, skcentsel, skmesh, skcent, skctrl;

-- rollout about :)
rollout rollaboutskw "About Smart SKEWer 1.1"
(
	label ab1 "Smart SKEWer 1.1"
	bitmap logobitmap filename:"sm_logo.jpg"
	label ab2 "by Jarek Dukat"
	label ab3 "http://www.nautilus.com.pl/jarek"
	label ab4 "jarek@nautilus.com.pl"
)

utility SmartSkewer "Smart SKEWer 1.1"
(
--------- utility layout
	group "Object Pickers"
	(
		label lab1 "Mesh or NURBS to Skew:"
		pickbutton skewmesh "Pick Object" filter:validmesh width:140 height:19

		label lab2 "Skew Center:" offset:[0,2]
		pickbutton skewcent "Pick Object" width:140 height:19

		label lab3 "Skew Controller:" offset:[0,2]
		pickbutton skewctrl "Pick object" width:140 height:19
	)

	group "Skew Options"
	(
		radiobuttons skewrel "Coordinate Mode:" labels:#("Relative (to mesh)","Absolute (to world)") default:1
		checkbox skewlnk1 "Link CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox skewmov1 "Move CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox skewmov2 "Move CenterObj to [0,0,0]" checked:true enabled:false offset:[-3,0]
		checkbox skewlnk2 "Link ControlObj to Mesh" checked:true offset:[-3,0]
		label lab4 "Skew Axis:" across:2 offset:[-12,0]
		radiobuttons skewax labels:#("X","Y","Z") default:3 columns:3 offset:[-6,0]
	)

	group "Skew Manager"
	(
		button skewapply "Apply Skew" offset:[0,1] height:20 width:100 enabled:false
		spinner skewscale "Skew Scale:" range:[0.01,100,1] fieldwidth:35 align:#center enabled:false
		button skewrefr "Refresh Skew" offset:[0,1] height:20 width:100 enabled:false
		button skewdel "Remove Skew" tooltip:"Remove Skew handler" height:20 width:100 enabled:false offset:[0,5]
		button skewfreeze "Freeze Skew" tooltip:"Create keyframes for Skew modifier (if requested) and remove Skew handler" height:20 width:100 enabled:false offset:[0,3]
		checkbox skewkey "Create Keyframes" checked:true enabled:false align:#center
		spinner skewkeyst "Start Frame: " range:[animationRange.start,animationRange.end,animationRange.start] fieldwidth:40 type:#integer enabled:false
		spinner skewkeyend "End Frame: " range:[animationRange.start,animationRange.end,animationRange.end] fieldwidth:40 type:#integer enabled:false
		spinner skewkeyby "Every n Frames: " range:[1,animationRange.end,5] fieldwidth:40 type:#integer enabled:false
	)

	checkbox skewexitfree "Remove Skew on Close" checked:true align:#center offset:[0,5]

--------- utility handlers
-- pick mesh object handler
	on skewmesh picked node do
	(
		skmeshsel = true
		if skctrlsel do
			if node.name == skctrl.name do
			(
				messagebox "This object has already been picked for Skew control,\npick another object" title:"Warning"
				skmeshsel = false
			)
		if skcentsel do
			if node.name == skcent.name do
			(
				messagebox "This object has already been picked for Skew center,\npick another object" title:"Warning"
				skmeshsel = false
			)
		if skmeshsel do
		(
			skewmesh.text = node.name
			skmesh = node
			enableskew()
		)
	)

-- pick skew center object handler
	on skewcent picked node do
	(
		skcentsel = true
		if skctrlsel do
			if node.name == skctrl.name do
			(
				messagebox "This object has already been picked for Skew control,\npick another object" title:"Warning"
				skcentsel = false
			)
		if skmeshsel do
			if node.name == skmesh.name do
			(
				messagebox "This object has already been picked for Skewing,\npick another object" title:"Warning"
				skcentsel = false
			)
		if skcentsel do
		(
			skewcent.text = node.name
			skcent = node
			enableskew()
		)
	)

-- pick skew control object handler
	on skewctrl picked node do
	(
		skctrlsel = true
		if skcentsel do
			if node.name == skcent.name do
			(
				messagebox "This object has already been picked for Skew center,\npick another object" title:"Warning"
				skctrlsel = false
			)
		if skmeshsel do
			if node.name == skmesh.name do
			(
				messagebox "This object has already been picked for Skewing,\npick another object" title:"Warning"
				skctrlsel = false
			)
		if skctrlsel do
		(
			skewctrl.text = node.name
			skctrl = node
			enableskew()
		)
	)

-- relative/absolute handler
	on skewrel changed sel do
		if sel == 1 then
		(
			skewlnk1.enabled = skewlnk2.enabled = skewmov1.enabled = true
			skewmov2.enabled = false
		)
		else
		(
			skewlnk1.enabled = skewlnk2.enabled = skewmov1.enabled = false
			skewmov2.enabled = true
		)
	
-- keyframes on/off handler
	on skewkey changed key do
		if key then
		(
			SmartSkewer.skewkeyst.enabled = true
			SmartSkewer.skewkeyend.enabled = true
			SmartSkewer.skewkeyby.enabled = true
		)
		else
		(
			SmartSkewer.skewkeyst.enabled = false
			SmartSkewer.skewkeyend.enabled = false
			SmartSkewer.skewkeyby.enabled = false
		)

-- skew scale refresh handler
	on skewscale changed scl do
		if SmartSkewer.skewrel.state == 1 then sk_refresh_amount skctrl.pos bm:skmesh.pos
		else sk_refresh_amount skctrl.pos

-- skew refresh button handler
	on skewrefr pressed do refreshskew()

-- call apply skew function
	on skewapply pressed do applyskew()
	
-- remove skew handler
	on skewdel pressed do deleteskewhandler()
	
-- freeze skew
	on skewfreeze pressed do freezeskew()

-- script autostart
	on SmartSkewer open do
	(
		skmeshsel = false; skcentsel = false; skctrlsel = false
		skewev_dir = undefined; skewev_cent = undefined
		skewev_amount = undefined; skewdev_mesh = undefined
		skmesh = undefined; skcent = undefined; skctrl = undefined
		addrollout rollaboutskw rolledup:true
	)

-- script close handler
	on SmartSkewer close do
	(
		if skewexitfree.checked do deleteskewhandler()
		removerollout rollaboutskw
	)
)

fn enableskew =
	if skmeshsel and skcentsel and skctrlsel do
		SmartSkewer.skewapply.enabled = true

--------- most important part, skew HANDLERS
fn sk_refresh_cent bc bm:[0,0,0] = skmesh.SmartSkew.center = bc - bm
fn sk_refresh_amount bc bm:[0,0,0] = skmesh.SmartSkew.amount = sqrt ( (bc.x - bm.x) ^ 2 + ( bc.y - bm.y ) ^ 2 ) * SmartSkewer.skewscale.value
fn sk_refresh_dir bc bm:[0,0,0] =
	skmesh.SmartSkew.direction = if (bc.x - bm.x) > 0 then
			-( asin ( (bc.y - bm.y) / sqrt ( (bc.x - bm.x) ^ 2 + (bc.y - bm.y) ^ 2)))
		else asin ( (bc.y - bm.y) / sqrt ( (bc.x - bm.x) ^ 2 + (bc.y - bm.y) ^ 2) ) + 180

-- this if a pick mesh filter
fn validmesh obj = superClassOf obj == GeometryClass

-- change buttons' state when skew handler removed
fn changeskewstate =
(
	SmartSkewer.skewapply.enabled = true
	SmartSkewer.skewdel.enabled = false
	SmartSkewer.skewrefr.enabled = false
	SmartSkewer.skewkey.enabled = false
	SmartSkewer.skewfreeze.enabled = false
	SmartSkewer.skewscale.enabled = false
	SmartSkewer.skewkeyst.enabled = false
	SmartSkewer.skewkeyend.enabled = false
	SmartSkewer.skewkeyby.enabled = false
	SmartSkewer.skewax.enabled = true
	SmartSkewer.skewrel.enabled = true
	if SmartSkewer.skewrel.state == 1 then
	(
		SmartSkewer.skewlnk1.enabled = true; SmartSkewer.skewmov1.enabled = true
		SmartSkewer.skewmov2.enabled = false; SmartSkewer.skewlnk2.enabled = true
	)
	else
	(
		SmartSkewer.skewlnk1.enabled = false; SmartSkewer.skewmov1.enabled = false
		SmartSkewer.skewmov2.enabled = true; SmartSkewer.skewlnk2.enabled = false
	)
)

-- refresh skew handler in current time
fn refreshskew =
(
	if SmartSkewer.skewrel.state == 1 then
		(sk_refresh_cent skcent.pos bm:skmesh.pos; sk_refresh_amount skctrl.pos bm:skmesh.pos; sk_refresh_dir skctrl.pos bm:skmesh.pos)
	else
		(sk_refresh_cent skcent.pos; sk_refresh_amount skctrl.pos; sk_refresh_dir skctrl.pos)
)

-- delete skew handler
fn deleteskewhandler =
(
	if skewev_cent != undefined do
	(
		deletechangehandler skewev_cent
		deletechangehandler skewev_amount
		deletechangehandler skewev_dir
		deletechangehandler skewdev_mesh
		deletemodifier skmesh (skmesh.modifiers["SmartSkew"])
		skewev_cent = undefined
		changeskewstate()
	)
)

-- freeze skew handler
fn freezeskew =
(
	deletechangehandler skewev_cent
	deletechangehandler skewev_amount
	deletechangehandler skewev_dir
	deletechangehandler skewdev_mesh
	mod2= skmesh.modifiers["SmartSkew"]
	mod2.enabledinviews = false
	befix = 1
	for i = 1 to skmesh.modifiers.count do if (mod2 == skmesh.modifiers[i]) do (befix = i; exit)
	addmodifier skmesh (skew name:"FreezedSmartSkew" axis:skmesh.SmartSkew.axis limit:skmesh.SmartSkew.limit upperlimit:skmesh.SmartSkew.upperlimit lowerlimit:skmesh.SmartSkew.lowerlimit) before:befix
	mod1= skmesh.modifiers["FreezedSmartSkew"]
	if SmartSkewer.skewkey.checked then
	(
		animate on
			for tm = SmartSkewer.skewkeyst.value to SmartSkewer.skewkeyend.value by SmartSkewer.skewkeyby.value do
				at time tm
				(
					slidertime = tm
					refreshskew()
					mod1.center = mod2.center
					mod1.amount = mod2.amount
					mod1.direction = mod2.direction
				)
		if SmartSkewer.skewkeyst.value != animationRange.start do
		(
			deletekey mod1.center.keys 1
			deletekey mod1.amount.keys 1
			deletekey mod1.direction.keys 1
		)
	)
	else
	(
		mod1.center = mod2.center
		mod1.amount = mod2.amount
		mod1.direction = mod2.direction
	)
	deletemodifier skmesh (mod2)
	skewev_cent = undefined
	changeskewstate()
)

-- this function applies the skew handlers and modifier
fn applyskew =
(
-- link and move helper objects depending on coordinate mode: relative/absolute
	if SmartSkewer.skewrel.state == 1 then
	(
		if SmartSkewer.skewmov1.checked do skcent.pos = skmesh.pos
		if SmartSkewer.skewlnk1.checked do skcent.parent = skmesh
		if SmartSkewer.skewlnk2.checked do skctrl.parent = skmesh
	)
	else if SmartSkewer.skewmov2.checked do skcent.pos = [0,0,0]

-- add modifier
	addmodifier skmesh (skew name:"SmartSkew" axis:SmartSkewer.skewax.state)
	
-- apply skew handler
	if SmartSkewer.skewrel.state == 1 then
	(
-- relative
		sk_refresh_cent skcent.pos bm:skmesh.pos; sk_refresh_amount skctrl.pos bm:skmesh.pos; sk_refresh_dir skctrl.pos bm:skmesh.pos
		skewev_cent = when transform skcent changes do sk_refresh_cent skcent.pos bm:skmesh.pos
		skewev_amount = when transform skctrl changes do sk_refresh_amount skctrl.pos bm:skmesh.pos
		skewev_dir = when transform skctrl changes do sk_refresh_dir skctrl.pos bm:skmesh.pos
		skewdev_mesh = when transform skmesh changes do
		(
			sk_refresh_cent skcent.pos bm:skmesh.pos; sk_refresh_amount skctrl.pos bm:skmesh.pos; sk_refresh_dir skctrl.pos bm:skmesh.pos
		)
	)
	else
	(
-- absolute
		sk_refresh_cent skcent.pos; sk_refresh_amount skctrl.pos; sk_refresh_dir skctrl.pos
		skewev_cent = when transform skcent changes do sk_refresh_cent skcent.pos
		skewev_amount = when transform skctrl changes do sk_refresh_amount skctrl.pos
		skewev_dir = when transform skctrl changes do sk_refresh_dir skctrl.pos
		skewdev_mesh = when transform skmesh changes do
		(
			sk_refresh_cent skcent.pos; sk_refresh_amount skctrl.pos; sk_refresh_dir skctrlskctrl.pos
		)
	)
	
-- change buttons state
	SmartSkewer.skewapply.enabled = false
	SmartSkewer.skewscale.enabled = true
	SmartSkewer.skewrefr.enabled = true
	SmartSkewer.skewdel.enabled = true
	SmartSkewer.skewkey.enabled = true
	SmartSkewer.skewfreeze.enabled = true
	SmartSkewer.skewrel.enabled = false
	SmartSkewer.skewlnk1.enabled = false; SmartSkewer.skewmov1.enabled = false
	SmartSkewer.skewmov2.enabled = false; SmartSkewer.skewlnk2.enabled = false
	SmartSkewer.skewax.enabled = false
	if SmartSkewer.skewkey.checked do
	(
		SmartSkewer.skewkeyst.enabled = true
		SmartSkewer.skewkeyend.enabled = true
		SmartSkewer.skewkeyby.enabled = true
	)
)
