-- Smart TAPERer 1.1
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This tool lets you use Taper modifier in the most interactive way you can imagine
-- you can control taper center and amount using two helper objects.

-- Please report any bugs or wishes by email

-- some global varialbes definition
global taperdev_mesh, taperev_cent, taperev_amount, tapmeshsel, tapctrlsel, tapcentsel, tapmesh, tapcent, tapctrl, tapsign;

-- rollout about :)
rollout rollabouttap "About Smart TAPERer 1.1"
(
	label ab1 "Smart TAPERer 1.1"
	bitmap logobitmap filename:"sm_logo.jpg"
	label ab2 "by Jarek Dukat"
	label ab3 "http://www.nautilus.com.pl/jarek"
	label ab4 "jarek@nautilus.com.pl"
)

utility smarttaperer "Smart TAPERer 1.1"
(
--------- utility layout
	group "Object Pickers"
	(
		label lab1 "Mesh or NURBS to Taper:"
		pickbutton tapermesh "Pick Object" filter:validmesh width:140 height:19

		label lab2 "Taper Center:" offset:[0,2]
		pickbutton tapercent "Pick Object" width:140 height:19

		label lab3 "Taper Controller:" offset:[0,2]
		pickbutton taperctrl "Pick object" width:140 height:19
	)

	group "Taper Options"
	(
		radiobuttons taperrel "Coordinate Mode:" labels:#("Relative (to mesh)","Absolute (to world)") default:1
		checkbox taperlnk1 "Link CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox tapermov1 "Move CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox tapermov2 "Move CenterObj to [0,0,0]" checked:true enabled:false offset:[-3,0]
		checkbox taperlnk2 "Link ControlObj to Mesh" checked:true offset:[-3,0]
		label lab4 "Prim. Axis:" across:2 offset:[-12,0]
		radiobuttons taperax labels:#("X","Y","Z") default:3 columns:3 offset:[-6,0]
		radiobuttons taperax1 "Effect Axis:" labels:#("X","Y","XY") default:3 columns:3 allign:#left offset:[-7,-5]
	)

	group "Taper Manager"
	(
		button taperapply "Apply Taper" offset:[0,1] height:20 width:100 enabled:false
		spinner taperscale "Taper Scale:" range:[0.01,100,1] scale:0.05 fieldwidth:35 align:#center enabled:false
		checkbox tapersym "Symmetry" checked:false enabled:false align:#left across:2 offset:[0,5]
		radiobuttons taperplus labels:#("+","-") default:1 columns:2 align:#right offset:[0,5] enabled:false
		button taperrefr "Refresh Taper" offset:[0,1] height:20 width:100 enabled:false
		button taperdel "Remove Taper" tooltip:"Remove taper handler" height:20 width:100 enabled:false offset:[0,5]
		button taperfreeze "Freeze Taper" tooltip:"Create keyframes for taper modifier (if requested) and remove taper handler" height:20 width:100 enabled:false offset:[0,3]
		checkbox taperkey "Create Keyframes" checked:true enabled:false align:#center
		spinner taperkeyst "Start Frame: " range:[animationRange.start,animationRange.end,animationRange.start] fieldwidth:40 type:#integer enabled:false
		spinner taperkeyend "End Frame: " range:[animationRange.start,animationRange.end,animationRange.end] fieldwidth:40 type:#integer enabled:false
		spinner taperkeyby "Every n Frames: " range:[1,animationRange.end,5] fieldwidth:40 type:#integer enabled:false
	)

	checkbox taperexitfree "Remove Taper on Close" checked:true align:#center offset:[0,5]

--------- utility handlers
-- pick mesh object handler
	on tapermesh picked node do
	(
		tapmeshsel = true
		if tapctrlsel do
			if node.name == tapctrl.name do
			(
				messagebox "This object has already been picked for taper control,\npick another object" title:"Warning"
				tapmeshsel = false
			)
		if tapcentsel do
			if node.name == tapcent.name do
			(
				messagebox "This object has already been picked for taper center,\npick another object" title:"Warning"
				tapmeshsel = false
			)
		if tapmeshsel do
		(
			tapermesh.text = node.name
			tapmesh = node
			enabletaper()
		)
	)

-- pick taper center object handler
	on tapercent picked node do
	(
		tapcentsel = true
		if tapctrlsel do
			if node.name == tapctrl.name do
			(
				messagebox "This object has already been picked for taper control,\npick another object" title:"Warning"
				tapcentsel = false
			)
		if tapmeshsel do
			if node.name == tapmesh.name do
			(
				messagebox "This object has already been picked for tapering,\npick another object" title:"Warning"
				tapcentsel = false
			)
		if tapcentsel do
		(
			tapercent.text = node.name
			tapcent = node
			enabletaper()
		)
	)

-- pick taper control object handler
	on taperctrl picked node do
	(
		tapctrlsel = true
		if tapcentsel do
			if node.name == tapcent.name do
			(
				messagebox "This object has already been picked for taper center,\npick another object" title:"Warning"
				tapctrlsel = false
			)
		if tapmeshsel do
			if node.name == tapmesh.name do
			(
				messagebox "This object has already been picked for tapering,\npick another object" title:"Warning"
				tapctrlsel = false
			)
		if tapctrlsel do
		(
			taperctrl.text = node.name
			tapctrl = node
			enabletaper()
		)
	)

-- relative/absolute handler
	on taperrel changed sel do
		if sel == 1 then
		(
			taperlnk1.enabled = taperlnk2.enabled = tapermov1.enabled = true
			tapermov2.enabled = false
		)
		else
		(
			taperlnk1.enabled = taperlnk2.enabled = tapermov1.enabled = false
			tapermov2.enabled = true
		)

-- relative/absolute handler
	on taperplus changed sig do
	(
		if sig == 1 then tapsign = 0.05 else tapsign = -0.05
		refreshtaper()
	)

-- keyframes on/off handler
	on taperkey changed key do
		if key then
		(
			smarttaperer.taperkeyst.enabled = true
			smarttaperer.taperkeyend.enabled = true
			smarttaperer.taperkeyby.enabled = true
		)
		else
		(
			smarttaperer.taperkeyst.enabled = false
			smarttaperer.taperkeyend.enabled = false
			smarttaperer.taperkeyby.enabled = false
		)

-- taper scale refresh handler
	on taperscale changed scl do
		if smarttaperer.taperrel.state == 1 then tap_refresh_amount tapctrl.pos bm:tapmesh.pos
		else tap_refresh_amount tapctrl.pos

-- taper symmetry refresh handler
	on tapersym changed sym do
		tapmesh.smarttaper.symmetry = smarttaperer.tapersym.checked

-- taper refresh button handler
	on taperrefr pressed do refreshtaper()

-- call apply taper function
	on taperapply pressed do applytaper()
	
-- remove taper handler
	on taperdel pressed do deletetaperhandler()
	
-- freeze taper
	on taperfreeze pressed do freezetaper()

-- script autostart
	on smarttaperer open do
	(
		tapmeshsel = false; tapcentsel = false; tapctrlsel = false
		taperev_cent = undefined; taperev_amount = undefined
		taperdev_mesh = undefined; tapmesh = undefined; tapcent = undefined
		tapctrl = undefined; tapsign = 0.05
		addrollout rollabouttap rolledup:true
	)

-- script close handler
	on smarttaperer close do
	(
		if taperexitfree.checked do deletetaperhandler()
		removerollout rollabouttap
	)
)

fn enabletaper =
	if tapmeshsel and tapcentsel and tapctrlsel do
		smarttaperer.taperapply.enabled = true

--------- most important part, TAPER HANDLERS
fn tap_refresh_cent bc bm:[0,0,0] = tapmesh.smarttaper.center = bc - bm
fn tap_refresh_amount bc bm:[0,0,0] = tapmesh.smarttaper.amount = sqrt ( (bc.x - bm.x) ^ 2 + ( bc.y - bm.y ) ^ 2 ) * smarttaperer.taperscale.value * tapsign

-- this if a pick mesh filter
fn validmesh obj = superClassOf obj == GeometryClass

-- change buttons' state when taper handler removed
fn changetaperstate =
(
	smarttaperer.taperapply.enabled = true
	smarttaperer.taperdel.enabled = false
	smarttaperer.taperrefr.enabled = false
	smarttaperer.taperkey.enabled = false
	smarttaperer.taperfreeze.enabled = false
	smarttaperer.taperscale.enabled = false
	smarttaperer.taperkeyst.enabled = false
	smarttaperer.taperkeyend.enabled = false
	smarttaperer.taperkeyby.enabled = false
	smarttaperer.taperax.enabled = true
	smarttaperer.taperax1.enabled = true
	smarttaperer.tapersym.enabled = false
	smarttaperer.taperplus.enabled = false
	smarttaperer.taperrel.enabled = true
	if smarttaperer.taperrel.state == 1 then
	(
		smarttaperer.taperlnk1.enabled = true; smarttaperer.tapermov1.enabled = true
		smarttaperer.tapermov2.enabled = false; smarttaperer.taperlnk2.enabled = true
	)
	else
	(
		smarttaperer.taperlnk1.enabled = false; smarttaperer.tapermov1.enabled = false
		smarttaperer.tapermov2.enabled = true; smarttaperer.taperlnk2.enabled = false
	)
)

-- refresh taper handler in current time
fn refreshtaper =
(
	if smarttaperer.taperrel.state == 1 then
		(tap_refresh_cent tapcent.pos bm:tapmesh.pos; tap_refresh_amount tapctrl.pos bm:tapmesh.pos)
	else
		(tap_refresh_cent tapcent.pos; tap_refresh_amount tapctrl.pos)
)

-- delete taper handler
fn deletetaperhandler =
(
	if taperev_cent != undefined do
	(
		deletechangehandler taperev_cent
		deletechangehandler taperev_amount
		deletechangehandler taperdev_mesh
		deletemodifier tapmesh (tapmesh.modifiers["smarttaper"])
		taperev_cent = undefined
		changetaperstate()
	)
)

-- freeze taper handler
fn freezetaper =
(
	deletechangehandler taperev_cent
	deletechangehandler taperev_amount
	deletechangehandler taperdev_mesh
	mod2= tapmesh.modifiers["SmartTaper"]
	mod2.enabledinviews = false
	befix = 1
	for i = 1 to tapmesh.modifiers.count do if (mod2 == tapmesh.modifiers[i]) do (befix = i; exit)
	addmodifier tapmesh (taper name:"FreezedSmartTaper" primaryaxis:tapmesh.smarttaper.primaryaxis effectaxis:tapmesh.smarttaper.effectaxis limit:tapmesh.smarttaper.limit upperlimit:tapmesh.smarttaper.upperlimit lowerlimit:tapmesh.smarttaper.lowerlimit curve:tapmesh.smarttaper.curve symmetry:tapmesh.smarttaper.symmetry) before:befix
	mod1= tapmesh.modifiers["FreezedSmartTaper"]
	if smarttaperer.taperkey.checked then
	(
		animate on
			for tm = smarttaperer.taperkeyst.value to smarttaperer.taperkeyend.value by smarttaperer.taperkeyby.value do
				at time tm
				(
					slidertime = tm
					refreshtaper()
					mod1.center = mod2.center
					mod1.amount = mod2.amount
				)
		if smarttaperer.taperkeyst.value != animationRange.start do
		(
			deletekey mod1.center.keys 1
			deletekey mod1.amount.keys 1
		)
	)
	else
	(
		mod1.center = mod2.center
		mod1.amount = mod2.amount
	)
	deletemodifier tapmesh (mod2)
	taperev_cent = undefined
	changetaperstate()
)

-- this function applies the taper handlers and modifier
fn applytaper =
(
-- link and move helper objects depending on coordinate mode: relative/absolute
	if smarttaperer.taperrel.state == 1 then
	(
		if smarttaperer.tapermov1.checked do tapcent.pos = tapmesh.pos
		if smarttaperer.taperlnk1.checked do tapcent.parent = tapmesh
		if smarttaperer.taperlnk2.checked do tapctrl.parent = tapmesh
	)
	else if smarttaperer.tapermov2.checked do tapcent.pos = [0,0,0]

-- add modifier
	addmodifier tapmesh (taper name:"SmartTaper" primaryaxis:(smarttaperer.taperax.state - 1) effectaxis:(smarttaperer.taperax1.state - 1) symmetry:smarttaperer.tapersym.checked)
	
-- apply taper handler
	if smarttaperer.taperrel.state == 1 then
	(
-- relative
		tap_refresh_cent tapcent.pos bm:tapmesh.pos; tap_refresh_amount tapctrl.pos bm:tapmesh.pos
		taperev_cent = when transform tapcent changes do tap_refresh_cent tapcent.pos bm:tapmesh.pos
		taperev_amount = when transform tapctrl changes do tap_refresh_amount tapctrl.pos bm:tapmesh.pos
		taperdev_mesh = when transform tapmesh changes do
		(
			tap_refresh_cent tapcent.pos bm:tapmesh.pos; tap_refresh_amount tapctrl.pos bm:tapmesh.pos
		)
	)
	else
	(
-- absolute
		tap_refresh_cent tapcent.pos; tap_refresh_amount tapctrl.pos
		taperev_cent = when transform tapcent changes do tap_refresh_cent tapcent.pos
		taperev_amount = when transform tapctrl changes do tap_refresh_amount tapctrl.pos
		taperdev_mesh = when transform tapmesh changes do
		(
			tap_refresh_cent tapcent.pos; tap_refresh_amount tapctrl.pos
		)
	)
	
-- change buttons state
	smarttaperer.taperapply.enabled = false
	smarttaperer.taperscale.enabled = true
	smarttaperer.taperrefr.enabled = true
	smarttaperer.taperdel.enabled = true
	smarttaperer.taperkey.enabled = true
	smarttaperer.taperfreeze.enabled = true
	smarttaperer.taperrel.enabled = false
	smarttaperer.taperlnk1.enabled = false; smarttaperer.tapermov1.enabled = false
	smarttaperer.tapermov2.enabled = false; smarttaperer.taperlnk2.enabled = false
	smarttaperer.taperax.enabled = false
	smarttaperer.taperax1.enabled = false
	smarttaperer.tapersym.enabled = true
	smarttaperer.taperplus.enabled = true
	if smarttaperer.taperkey.checked do
	(
		smarttaperer.taperkeyst.enabled = true
		smarttaperer.taperkeyend.enabled = true
		smarttaperer.taperkeyby.enabled = true
	)
)
