-- Smart TWISTer 1.1
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

-- This tool lets you use Twist modifier in the most interactive way you can imagine
-- you can control twist center and amount using two helper objects.

-- Please report any bugs or wishes by email

-- some global varialbes definition
global twistdev_mesh, twistev_cent, twistev_angle, twmeshsel, twctrlsel, twcentsel, twmesh, twcent, twctrl;

-- rollout about :)
rollout rollabouttws "About Smart TWISTer 1.1"
(
	label ab1 "Smart TWISTer 1.1"
	bitmap logobitmap filename:"sm_logo.jpg"
	label ab2 "by Jarek Dukat"
	label ab3 "http://www.nautilus.com.pl/jarek"
	label ab4 "jarek@nautilus.com.pl"
)

utility smarttwister "Smart TWISTer 1.1"
(
--------- utility layout
	group "Object Pickers"
	(
		label lab1 "Mesh or NURBS to Twist:"
		pickbutton twistmesh "Pick Object" filter:validmesh width:140 height:19

		label lab2 "Twist Center:" offset:[0,2]
		pickbutton twistcent "Pick Object" width:140 height:19

		label lab3 "Twist Controller:" offset:[0,2]
		pickbutton twistctrl "Pick object" width:140 height:19
	)

	group "Twist Options"
	(
		radiobuttons twistrel "Coordinate Mode:" labels:#("Relative (to mesh)","Absolute (to world)") default:1
		checkbox twistlnk1 "Link CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox twistmov1 "Move CenterObj to Mesh" checked:true offset:[-3,0]
		checkbox twistmov2 "Move CenterObj to [0,0,0]" checked:true enabled:false offset:[-3,0]
		checkbox twistlnk2 "Link ControlObj to Mesh" checked:true offset:[-3,0]
		label lab4 "Twist Axis:" across:2 offset:[-12,0]
		radiobuttons twistax labels:#("X","Y","Z") default:3 columns:3 offset:[-6,0]
	)

	group "Twist Manager"
	(
		button twistapply "Apply Twist" offset:[0,1] height:20 width:100 enabled:false
		spinner twistscale "Twist Scale:" range:[0.01,100,1] fieldwidth:35 align:#center enabled:false
		button twistrefr "Refresh Twist" offset:[0,1] height:20 width:100 enabled:false
		button twistdel "Remove Twist" tooltip:"Remove twist handler" height:20 width:100 enabled:false offset:[0,5]
		button twistfreeze "Freeze Twist" tooltip:"Create keyframes for twist modifier (if requested) and remove twist handler" height:20 width:100 enabled:false offset:[0,3]
		checkbox twistkey "Create Keyframes" checked:true enabled:false align:#center
		spinner twistkeyst "Start Frame: " range:[animationRange.start,animationRange.end,animationRange.start] fieldwidth:40 type:#integer enabled:false
		spinner twistkeyend "End Frame: " range:[animationRange.start,animationRange.end,animationRange.end] fieldwidth:40 type:#integer enabled:false
		spinner twistkeyby "Every n Frames: " range:[1,animationRange.end,5] fieldwidth:40 type:#integer enabled:false
	)

	checkbox twistexitfree "Remove Twist on Close" checked:true align:#center offset:[0,5]

--------- utility handlers
-- pick mesh object handler
	on twistmesh picked node do
	(
		twmeshsel = true
		if twctrlsel do
			if node.name == twctrl.name do
			(
				messagebox "This object has already been picked for twist control,\npick another object" title:"Warning"
				twmeshsel = false
			)
		if twcentsel do
			if node.name == twcent.name do
			(
				messagebox "This object has already been picked for twist center,\npick another object" title:"Warning"
				twmeshsel = false
			)
		if twmeshsel do
		(
			twistmesh.text = node.name
			twmesh = node
			enabletwist()
		)
	)

-- pick twist center object handler
	on twistcent picked node do
	(
		twcentsel = true
		if twctrlsel do
			if node.name == twctrl.name do
			(
				messagebox "This object has already been picked for twist control,\npick another object" title:"Warning"
				twcentsel = false
			)
		if twmeshsel do
			if node.name == twmesh.name do
			(
				messagebox "This object has already been picked for twisting,\npick another object" title:"Warning"
				twcentsel = false
			)
		if twcentsel do
		(
			twistcent.text = node.name
			twcent = node
			enabletwist()
		)
	)

-- pick twist control object handler
	on twistctrl picked node do
	(
		twctrlsel = true
		if twcentsel do
			if node.name == twcent.name do
			(
				messagebox "This object has already been picked for twist center,\npick another object" title:"Warning"
				twctrlsel = false
			)
		if twmeshsel do
			if node.name == twmesh.name do
			(
				messagebox "This object has already been picked for twisting,\npick another object" title:"Warning"
				twctrlsel = false
			)
		if twctrlsel do
		(
			twistctrl.text = node.name
			twctrl = node
			enabletwist()
		)
	)

-- relative/absolute handler
	on twistrel changed sel do
		if sel == 1 then
		(
			twistlnk1.enabled = twistlnk2.enabled = twistmov1.enabled = true
			twistmov2.enabled = false
		)
		else
		(
			twistlnk1.enabled = twistlnk2.enabled = twistmov1.enabled = false
			twistmov2.enabled = true
		)
	
-- keyframes on/off handler
	on twistkey changed key do
		if key then
		(
			smarttwister.twistkeyst.enabled = true
			smarttwister.twistkeyend.enabled = true
			smarttwister.twistkeyby.enabled = true
		)
		else
		(
			smarttwister.twistkeyst.enabled = false
			smarttwister.twistkeyend.enabled = false
			smarttwister.twistkeyby.enabled = false
		)

-- twist scale refresh handler
	on twistscale changed scl do
		if smarttwister.twistrel.state == 1 then tw_refresh_angle twctrl.pos bm:twmesh.pos
		else tw_refresh_angle twctrl.pos

-- twist refresh button handler
	on twistrefr pressed do refreshtwist()

-- call apply twist function
	on twistapply pressed do applytwist()
	
-- remove twist handler
	on twistdel pressed do deletetwisthandler()
	
-- freeze twist
	on twistfreeze pressed do freezetwist()

-- script autostart
	on smarttwister open do
	(
		twmeshsel = false; twcentsel = false; twctrlsel = false; twistev_cent = undefined
		twistev_angle = undefined; twistdev_mesh = undefined
		twmesh = undefined; twcent = undefined; twctrl = undefined
		addrollout rollabouttws rolledup:true
	)

-- script close handler
	on smarttwister close do
	(
		if twistexitfree.checked do deletetwisthandler()
		removerollout rollabouttws
	)
)

fn enabletwist =
	if twmeshsel and twcentsel and twctrlsel do
		smarttwister.twistapply.enabled = true

--------- most important part, TWIST HANDLERS
fn tw_refresh_cent bc bm:[0,0,0] = twmesh.smarttwist.center = bc - bm
fn tw_refresh_angle bc bm:[0,0,0] = twmesh.smarttwist.angle = sqrt ( (bc.x - bm.x) ^ 2 + ( bc.y - bm.y ) ^ 2 ) * smarttwister.twistscale.value

-- this if a pick mesh filter
fn validmesh obj = superClassOf obj == GeometryClass

-- change buttons' state when twist handler removed
fn changetwiststate =
(
	smarttwister.twistapply.enabled = true
	smarttwister.twistdel.enabled = false
	smarttwister.twistrefr.enabled = false
	smarttwister.twistkey.enabled = false
	smarttwister.twistfreeze.enabled = false
	smarttwister.twistscale.enabled = false
	smarttwister.twistkeyst.enabled = false
	smarttwister.twistkeyend.enabled = false
	smarttwister.twistkeyby.enabled = false
	smarttwister.twistax.enabled = true
	smarttwister.twistrel.enabled = true
	if smarttwister.twistrel.state == 1 then
	(
		smarttwister.twistlnk1.enabled = true; smarttwister.twistmov1.enabled = true
		smarttwister.twistmov2.enabled = false; smarttwister.twistlnk2.enabled = true
	)
	else
	(
		smarttwister.twistlnk1.enabled = false; smarttwister.twistmov1.enabled = false
		smarttwister.twistmov2.enabled = true; smarttwister.twistlnk2.enabled = false
	)
)

-- refresh twist handler in current time
fn refreshtwist =
(
	if smarttwister.twistrel.state == 1 then
		(tw_refresh_cent twcent.pos bm:twmesh.pos; tw_refresh_angle twctrl.pos bm:twmesh.pos)
	else
		(tw_refresh_cent twcent.pos; tw_refresh_angle twctrl.pos)
)

-- delete twist handler
fn deletetwisthandler =
(
	if twistev_cent != undefined do
	(
		deletechangehandler twistev_cent
		deletechangehandler twistev_angle
		deletechangehandler twistdev_mesh
		deletemodifier twmesh (twmesh.modifiers["smarttwist"])
		twistev_cent = undefined
		changetwiststate()
	)
)

-- freeze twist handler
fn freezetwist =
(
	deletechangehandler twistev_cent
	deletechangehandler twistev_angle
	deletechangehandler twistdev_mesh
	mod2= twmesh.modifiers["SmartTwist"]
	mod2.enabledinviews = false
	befix = 1
	for i = 1 to twmesh.modifiers.count do if (mod2 == twmesh.modifiers[i]) do (befix = i; exit)
	addmodifier twmesh (twist name:"FreezedSmartTwist" bias:twmesh.smarttwist.bias axis:twmesh.smarttwist.axis limit:twmesh.smarttwist.limit upperlimit:twmesh.smarttwist.upperlimit lowerlimit:twmesh.smarttwist.lowerlimit) before:befix
	mod1= twmesh.modifiers["FreezedSmartTwist"]
	if smarttwister.twistkey.checked then
	(
		animate on
			for tm = smarttwister.twistkeyst.value to smarttwister.twistkeyend.value by smarttwister.twistkeyby.value do
				at time tm
				(
					slidertime = tm
					refreshtwist()
					mod1.center = mod2.center
					mod1.angle = mod2.angle
				)
		if smarttwister.twistkeyst.value != animationRange.start do
		(
			deletekey mod1.center.keys 1
			deletekey mod1.angle.keys 1
		)
	)
	else
	(
		mod1.center = mod2.center
		mod1.angle = mod2.angle
	)
	deletemodifier twmesh (mod2)
	twistev_cent = undefined
	changetwiststate()
)

-- this function applies the twist handlers and modifier
fn applytwist =
(
-- link and move helper objects depending on coordinate mode: relative/absolute
	if smarttwister.twistrel.state == 1 then
	(
		if smarttwister.twistmov1.checked do twcent.pos = twmesh.pos
		if smarttwister.twistlnk1.checked do twcent.parent = twmesh
		if smarttwister.twistlnk2.checked do twctrl.parent = twmesh
	)
	else if smarttwister.twistmov2.checked do twcent.pos = [0,0,0]

-- add modifier
	addmodifier twmesh (twist name:"SmartTwist" axis:(smarttwister.twistax.state - 1))
	
-- apply twist handler
	if smarttwister.twistrel.state == 1 then
	(
-- relative
		tw_refresh_cent twcent.pos bm:twmesh.pos; tw_refresh_angle twctrl.pos bm:twmesh.pos
		twistev_cent = when transform twcent changes do tw_refresh_cent twcent.pos bm:twmesh.pos
		twistev_angle = when transform twctrl changes do tw_refresh_angle twctrl.pos bm:twmesh.pos
		twistdev_mesh = when transform twmesh changes do
		(
			tw_refresh_cent twcent.pos bm:twmesh.pos; tw_refresh_angle twctrl.pos bm:twmesh.pos
		)
	)
	else
	(
-- absolute
		tw_refresh_cent twcent.pos; tw_refresh_angle twctrl.pos
		twistev_cent = when transform twcent changes do tw_refresh_cent twcent.pos
		twistev_angle = when transform twctrl changes do tw_refresh_angle twctrl.pos
		twistdev_mesh = when transform twmesh changes do
		(
			tw_refresh_cent twcent.pos; tw_refresh_angle twctrl.pos
		)
	)
	
-- change buttons state
	smarttwister.twistapply.enabled = false
	smarttwister.twistscale.enabled = true
	smarttwister.twistrefr.enabled = true
	smarttwister.twistdel.enabled = true
	smarttwister.twistkey.enabled = true
	smarttwister.twistfreeze.enabled = true
	smarttwister.twistrel.enabled = false
	smarttwister.twistlnk1.enabled = false; smarttwister.twistmov1.enabled = false
	smarttwister.twistmov2.enabled = false; smarttwister.twistlnk2.enabled = false
	smarttwister.twistax.enabled = false
	if smarttwister.twistkey.checked do
	(
		smarttwister.twistkeyst.enabled = true
		smarttwister.twistkeyend.enabled = true
		smarttwister.twistkeyby.enabled = true
	)
)
